(defun process-result (result)
  (format t "Final Score: ~A~%" result))
  
(defun choose-action (perceptions &aux (hasValidAction nil) (hasValidDirection nil) action direction actionRequest)
  ;; display the current perceptions
  (format t "~%Perceptions: ~A" perceptions)
  
  ;; loop to get the action
  (loop until (equal hasValidAction t) do 
	;; display the prompt
	(format t "~%What is your next action?~%")
	(format t "Moves:   (N)orth   (E)ast     (S)outh    (W)est~%")
	(format t "Actions: (P)ickup  (D)ropoff  (C)Sense   (-)NoOp~%")
	
	;; read the user's selection
	(setf action (read))
	
	;; parse and process the user's selection
	(setf hasValidAction t)  ; temporarily assume the action is valid
	(cond ((string-equal action "N")
	       (setf action "MOVE")
	       (setf direction "NORTH"))
	      ((string-equal action "S")
	       (setf action "MOVE")
	       (setf direction "SOUTH"))
	      ((string-equal action "E")
	       (setf action "MOVE")
	       (setf direction "EAST"))
	      ((string-equal action "W")
	       (setf action "MOVE")
	       (setf direction "WEST"))
	      ((string-equal action "P")
	       (setf action "PICKUP"))
	      ((string-equal action "D")
	       (setf action "DROPOFF"))
	      ((string-equal action "C")
	       (setf action "SENSE"))
	      ((string-equal action "-")
	       (setf action "NOOP")
	       (setf direction ""))
	      (t            ; if the user didn't input a valid action, reverse our assumption
	       (format t "~%Invalid action.~%")  
	       (setf hasValidAction nil))))
  
  ;; loop to get the direction to go with the action, if needed
  (if (and (not (string-equal action "NOOP")) (not (string-equal action "MOVE")))
      (loop until (equal hasValidDirection t) do
	
	    ;; prompt for the direction
	    (format t "~%What is the direction for this ~A action?~%" action)
	    (format t " (N)orth  (E)ast    (S)outh    (W)est~%")
	    
	    ;; read the user's choice
	    (setf direction (read))
	    
	    ;; parse and process the choice
	    (setf hasValidDirection t)
	    (cond ((string-equal direction "N") (setf direction "NORTH"))
		  ((string-equal direction "S") (setf direction "SOUTH"))	
		  ((string-equal direction "E") (setf direction "EAST"))
		  ((string-equal direction "W") (setf direction "WEST"))
		  (t (setf hasValidDirection nil)
		     (format t "~%Error:  invalid direction.~%")))))
  
  ;; generate the action request
  (setf actionRequest (concatenate 'string "<action> <name> "  action " </name> <direction> " direction " </direction> <time> " (write-to-string currentTime) " </time> </action> "))

  ;; echo the action we're requesting
  (format t "~%Requesting action: ~A~%" actionRequest)
  (incf currentTime)
  
  ;; return the selected action string
  actionRequest)



  

(defun start (connection &aux perception (terminate nil))
  (loop until (equal terminate t) do
	(setf perception (read-line connection))
	(format t "RAW: ~A~%" (get-word 1 perception))
	(cond ((string-equal (get-word 1 perception) "<finalscore>")
	       (process-result (get-word 2 perception))
	       (setf terminate t))
	      (t
	       (write-line (choose-action perception) connection)))))



(defun process-connection (connection agent-name)
  (write-line (concatenate 'string "register_agent " agent-name) connection)
  (start connection)
  (close connection))




(defun get-word (wordIdx string &aux nextSpacePosition)
  ;; error check the arguments
  (check-type wordIdx integer "an integer >= 1")
  (assert (>= wordIdx 1)
	  (wordIdx)
	  "wordIdx must be >= 1.")
  
  ;; get the position of the next space
  (setf nextSpacePosition (position #\Space string))
  
  ;; determine the desired word
  (cond ;; no more words, so return nil
   ((equal (length string) 0) 
    nil)
   ;; it's the next word, so return it
   ((equal wordIdx 1) 
    (subseq string 0 nextSpacePosition))
   ;; there aren't any more space and it's not the next word
   ((equal nextSpacePosition nil)
    nil)
   ;; otherwise, recurse to determine the desired word
   (t
    (get-word (- wordIdx 1) 
	      (subseq string (+ nextSpacePosition 1))))))




(defun main (host port agent-name)
  ;; error check the arguments
  (check-type host string "an IP address")
  (check-type port integer "an integer")
  (check-type agent-name string "a string representing the agent's name")

  (setf currenttime 0)
  
  (let ((connection (socket:socket-connect port host)))
    (process-connection connection agent-name)))
